# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that manages a single instance of striim.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

INTERVAL = 10

class InstanceManager(object):
	"""
	This class creates an object that manages a single striim instance.
	"""

	def __init__(self, argv):
		"""
		Initialize the InstanceManager.

		@param self: Reference to the InstanceManager instance.
		@type self: An InstanceManager object.

		@param argv: Command line arguments.
		@type argv: A List object.
		"""
		# Expose striim's service to D-Bus.
		from DBusService import DBusService
		DBusService(self)
		from gobject import idle_add, timeout_add, PRIORITY_LOW
		idle_add(self.__init_gnome_libs)
		from sys import setcheckinterval
		setcheckinterval(INTERVAL)
		self.__init_attributes()
		self.__init_i18n()
		self.__init_player(argv)
		# Call the garbage collector every 10 minutes.
		timeout_add(600000, self.__init_garbage_collector, priority=PRIORITY_LOW)

	def __init_attributes(self):
		"""
		Initialize data attributes.

		@param self: Reference to the InstanceManager instance.
		@type self: An InstanceManager object.
		"""
		self.__player = None
		return

	def show_player(self, argv):
		"""
		Reshow the striim and parse new playlist if available.

		@param self: Reference to the InstanceManager instance.
		@type self: An InstanceManager object.

		@param argv: A list of shoutcast playlist
		@type argv: A List object.
		"""
		self.__player.reshow_window()
		self.__player.parse_playlist(argv)
		return

	def __init_i18n(self):
		"""
		Prepare striim for internationalization.

		@param self: Reference to the InstanceManager instance.
		@type self: An InstanceManager object.
		"""
		from Global import striim_data_path
		from os import path
		locale_folder = path.join(striim_data_path, "locale")
		# Initialize glade first.
		try:
			from locale import setlocale, LC_ALL, Error, bindtextdomain
			bindtextdomain("striim", locale_folder)
			setlocale(LC_ALL, "")
		except Error:
			pass
		from gtk import glade
		glade.bindtextdomain("striim", locale_folder)
		glade.textdomain("striim")
		from gettext import textdomain, bindtextdomain
		from gettext import install, bind_textdomain_codeset
		bindtextdomain("striim", locale_folder)
		bind_textdomain_codeset("striim", "UTF-8")
		textdomain("striim")
		install("striim", locale_folder, unicode=1)
		return

	def __init_player(self, argv):
		"""
		Initialize the radio player.

		@param self: Reference to the InstanceManager instance.
		@type self: An InstanceManager object.

		@param argv: Possible command line arguments.
		@type argv: A List object.
		"""
		from Player import Player
		self.__player = Player(self, argv)
		return

	def __init_garbage_collector(self):
		"""
		Call the Python garbage collector.

		@param self: Reference to the EditorManager instance.
		@type self: An EditorManager object.
		"""
		from gc import collect
		collect()
		return True

	def __init_gnome_libs(self):
		"""
		Initialize the GNOME libraries.

		@param self: Reference to the InstanceManager instance.
		@type self: A InstanceManager object.
		"""
		from utils import init_gnome
		init_gnome()
		return False

	def quit(self):
		"""
		Quit striim.

		@param self: Reference to the InstanceManager instance.
		@type self: An InstanceManager object.
		"""
		from os import _exit
		_exit(0)
		return
