# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This modules contains a library of generic auxiliary functions that can be used
accross the project. The functions are designed with reuse and modularity in
mind.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

def calculate_resolution_independence(window, width, height):
	"""
	Calculate the resolution independent width and height of windows based on
	the host system's monitor resolution.

	@param window: Any window/dialog object.
	@type window: A gtk.Window object.

	@param width: An arbitrary ratio used to determine a resolution independent
		width.
	@type width: A Float/Integer object.

	@param height: An arbitrary ratio used to determine a resolution independent
		height.
	@type height: A Float/Integer object.

	@return: The width and height to set the window objects to.
	@rtype: A Tuple object containing the suggested width and height to set
		windows to.

	"""
	screen = window.get_screen()
	number = screen.get_number()
	rectangle = screen.get_monitor_geometry(number)
	width = int(rectangle.width/width)
	height = int(rectangle.height/height)
	return width, height

def generate_random_number(sequence):
	from random import random
	while True:
		exit = True
		number = random()
		if sequence:
			for item in sequence:
				if number == item:
					exit = False
					break
		if exit:
			break
	return number

from gobject import main_context_default
context = main_context_default()
pending = context.pending
iteration = context.iteration

def response():
	"""
	Allow scribes GUI to be responsive when performing tasks that can
	potentially	lock the GUI.
	"""
	global pending, iteration
	while pending():
		iteration(False)
	return False

def select_row(treeview, column=0):
	"""
	Select a row in a treeview if any.

	This function disables the treeview if it has no rows. Otherwise,
	it tries to figure out any row to select.

	@param treeview: Reference to a TreeView object.
	@type self: An gtk.TreeView object.
	"""
	selection = treeview.get_selection()
	model = treeview.get_model()
	path, iterator = selection.get_selected()
	if iterator:
		path = model.get_path(iterator)
		treeview.set_cursor(path, treeview.get_column(column))
		treeview.grab_focus()
	else:
		first_iterator = model.get_iter_first()
		if first_iterator:
			path = model.get_path(first_iterator)
			treeview.set_cursor(path, treeview.get_column(column))
			treeview.grab_focus()
		else:
			treeview.set_property("sensitive", False)
	return

def disconnect_signal(signal_id, instance):
	"""
	Disconnect a signal from an object.

	@param signal_id: A numeric identifier for a signal.
	@type signal_id: A Integer object.

	@param instance: An object connected to a signal.
	@type instance: A GObject object.
	"""
	try:
		from operator import truth
		if truth(signal_id) and truth(instance.handler_is_connected(signal_id)):
			instance.disconnect(signal_id)
	except AttributeError:
		pass
	return

def delete_attributes(instance):
	"""
	Delete "writable" attributes of an object.

	@param instance: An object to delete.
	@type instance: A Python object.
	"""
	attribute_names = dir(instance)
	for name in attribute_names:
		try:
			if name in ["allocation", "window"]:
				# Ignore "allocation" and "window" attributes. Deleting
				# either attributes crashes GTK+.
				continue
			delattr(instance, name)
		except AttributeError:
			pass
		except TypeError:
			pass
		except Exception:
			pass
	return

def delete_list(sequence):
	"""
	Delete a list and its content
	"""
	from operator import truth
	if truth(sequence):
		[sequence.remove(x) for x in sequence]
	return

def get_pixbuf_from_file(filename, size=None):
	"""
	Get pixbuf from a file.

	@param filename: Full path to an image file.
	@type filename: A String object.

	@param size: Size to scale the image to.
	@type size: A Tuple object.
	"""
	from gtk.gdk import pixbuf_new_from_file, INTERP_HYPER
	from operator import not_
	if not_(size): return pixbuf_new_from_file(filename)
	width, height = size
	pixbuf = pixbuf_new_from_file(filename).scale_simple(width, height, INTERP_HYPER)
	return pixbuf

def init_gnome():
	"""
	Initialize the GNOME libraries.
	"""
	from Global import version, name, striim_data_path
	from gnome import PARAM_APP_DATADIR, program_init, ui
	properties = {
		PARAM_APP_DATADIR: striim_data_path,
	}
	program = program_init(name, version, properties=properties)
	return

try:
	from psyco import bind
	bind(get_pixbuf_from_file)
	bind(response)
except ImportError:
	pass
